//
//	asock.h
//
//	Created on: Apr 28, 2022
//      Author: vagner
//

#pragma once


#include <string>
#include <mutex>

#include <vCommon.h>
#include <LogItem.h>

namespace vsock
{

    class asock : LogItem
    {

    public:

        //  ctor
        //  name - логическое имя
        //  rto - таймаут приема (msec)
        //  sto - таймаут передачи (msec)
        asock ( std::string &name, const std::string &Host, const int Port, int rOut, int sOut );

        //  dtor
        virtual ~asock ( );

        //  открыть соединение
        bool Open ( );

        //  закрыть соединение
        void Close ( );

        //  принять
        //  возвращает:
        //  <0  ошибка
        //   0  разрыв связи
        //  >0  размер принятого
        int Recv ( void *buf, int bSize );

        //  принять указанный размер
        bool RecvAll ( void *buf, int bSize );

        //  отправить
        //  возвращает:
        //  <0  ошибка
        //  >0  размер переданного
        int Send ( void *buf, int bSize );

        //  отправить указанный размер
        bool SendAll ( void *buf, int bSize );

        //  наличие соединения
        inline bool IsConnected ( )
        {
            return _IsConnected;
        }

        //  валидность сокета
        inline bool IsValidSocket ( )
        {
            return sockfd > 0;
        }

        //  сбросить приемный буфер
        void flush_input ( );

        //  set_timeout
        void set_timeout ( int rto, int sto );

        //  restart
        bool resatrt ( );

    private:

        //  сокет
        int sockfd;

        //  соединение установлено
        static bool _IsConnected;

        //  получено сообщение
        static bool _IsMessage;

        //  таймаут приема
        int rto;

        //  таймаут передачи
        int sto;

        //  host
        std::string Host;

        //  port
        int Port;

        static void* buf;
        static int bsize;

        //  семафор доступа к флагам
        std::mutex mxFlag;


        bool wait_for ( bool& flag, int to );

        int aReceive ( asock *socket, void* buf, int bsize );
        void aConnect ( asock *socket );

        //  получить значение bool
        bool get_bool ( const bool& flag );

        //  установить значение bool
        void set_bool ( bool& flag, bool value );

        //  опрос
        bool poll ( short int flag, int t_out );

    };

} /* namespace vsock */
